@@ -25,10 +25,19 @@ from .typing import (
     ChannelsType,
     ContextType,
     FavoriteListType,
+    MenuItemsSettingsCurrent,
+    MenuItemsSettingsCurrentPost,
+    MenuItemsSettingsCurrentValueValue,
+    MenuItemsSettingsEntry,
+    MenuItemsSettingsNode,
+    MenuItemsSettingsStructure,
+    MenuItemsSettingsUpdate,
+    MenuItemsSettingsValueData,
     SystemType,
     ApplicationType,
 )
 from .auth import CachedDigestAuth
+from httpcore import AsyncConnectionPool
 
 LOG = logging.getLogger(__name__)
 TIMEOUT = 20.0
@@ -168,6 +177,47 @@ class NoneJsonData(GeneralFailure):
 
 T = TypeVar("T")
 
+def track_removed_connections(session: httpx.AsyncClient):
+    """
+    httpcore version up to at least 1.13.7 series contain a bug where a session is leaked
+    if the server closes the connection, a pull request to solve the issue exists here
+    https://github.com/encode/httpcore/pull/117 but so far it's not been merged.
+
+    Version 1.14.x of httpx library is re-designed and is not yet known to suffer this bug,
+    it however might suffer another similar: https://github.com/encode/httpcore/issues/431
+    """
+    removed_connections = set()
+    
+    try:
+        pool: AsyncConnectionPool = session._transport._pools
+        original_add_to_pool = pool._add_to_pool
+        original_remove_from_pool = pool._remove_from_pool
+    except AttributeError:
+        LOG.debug("Object to patch is missing, maybe library have been upgraded so skipping")
+        return removed_connections
+
+    async def _add_to_pool(
+        connection, timeout
+    ) -> None:
+        if connection in removed_connections:
+            removed_connections.remove(connection)
+        await original_add_to_pool(connection, timeout)
+    pool._add_to_pool = _add_to_pool
+    async def _remove_from_pool(
+        connection
+    ) -> None:
+        removed_connections.add(connection)
+        await original_remove_from_pool(connection)
+    pool._remove_from_pool = _remove_from_pool
+    return removed_connections
+
+
+async def close_removed_connections(removed_connections: set):
+    for connection in removed_connections:
+        LOG.warning("Closing connection leaked from pool")
+        if connection.should_close():
+            await connection.aclose()
+    removed_connections.clear()
 
 class PhilipsTV(object):
     def __init__(
@@ -179,13 +229,14 @@ class PhilipsTV(object):
         password=None,
         verify=False,
         auth_shared_key=None,
+        system = None
     ):
         self._host = host
         self._connfail = 0
         self.api_version = int(api_version)
         self.on = False
         self.name: Optional[str] = None
-        self.system: Optional[SystemType] = None
+        self.system: Optional[SystemType] = system
         self.sources = {}
         self.source_id = None
         self.audio_volume = None
@@ -197,6 +248,8 @@ class PhilipsTV(object):
         self.application: Optional[ApplicationIntentType] = None
         self.context: Optional[ContextType] = None
         self.screenstate: Optional[str] = None
+        self.settings: Optional[MenuItemsSettingsStructure] = None
+        self.settings_version = 0
         self.ambilight_topology = None
         self.ambilight_mode_set = None
         self.ambilight_mode_raw: Optional[str] = None
@@ -208,13 +261,14 @@ class PhilipsTV(object):
         self.ambilight_current_configuration: Optional[
             AmbilightCurrentConfiguration
         ] = None
+        self.huelamp_power: Optional[str] = None
         self.powerstate = None
         if auth_shared_key:
             self.auth_shared_key = auth_shared_key
         else:
             self.auth_shared_key = AUTH_SHARED_KEY
 
-        if secured_transport:
+        if secured_transport or self.secured_transport:
             self.protocol = "https"
         else:
             self.protocol = "http"
@@ -224,6 +278,8 @@ class PhilipsTV(object):
         self.session = httpx.AsyncClient(limits=limits, timeout=timeout, verify=False)
         self.session.headers["Accept"] = "application/json"
 
+        self.removed_connections = track_removed_connections(self.session)
+
         if username and password:
             self.session.auth = CachedDigestAuth(username, password)
 
@@ -505,6 +561,8 @@ class PhilipsTV(object):
             raise ConnectionFailure(err) from err
         except httpx.HTTPError as err:
             raise GeneralFailure(err) from err
+        finally:
+            await close_removed_connections(self.removed_connections)
 
     async def _getBinary(self, path: str) -> Tuple[Optional[bytes], Optional[str]]:
 
@@ -520,6 +578,8 @@ class PhilipsTV(object):
             raise ConnectionFailure(err) from err
         except httpx.HTTPError as err:
             raise GeneralFailure(err) from err
+        finally:
+            await close_removed_connections(self.removed_connections)
 
     async def postReq(self, path: str, data: Dict, timeout=None, protocol=None) -> Optional[Dict]:
         try:
@@ -538,10 +598,12 @@ class PhilipsTV(object):
             return None
         except (httpx.ConnectTimeout, httpx.ConnectError) as err:
             raise ConnectionFailure(err) from err
-        except httpx.ProtocolError as err:
+        except (httpx.ProtocolError, httpx.ReadError) as err:
             raise ProtocolFailure(err) from err
         except httpx.HTTPError as err:
             raise GeneralFailure(err) from err
+        finally:
+            await close_removed_connections(self.removed_connections)
 
     async def pairRequest(
         self,
@@ -641,6 +703,7 @@ class PhilipsTV(object):
                 await self.getApplications()
                 await self.getAmbilightSupportedStyles()
                 await self.getAmbilightCached()
+                await self.getMenuItemsSettingsStructure()
 
             await self.getPowerState()
             await self.getAudiodata()
@@ -652,6 +715,7 @@ class PhilipsTV(object):
             await self.getAmbilightMode()
             await self.getAmbilightPower()
             await self.getAmbilightCurrentConfiguration()
+            await self.getHueLampPower()
             self.on = True
             return True
         except ConnectionFailure as err:
@@ -705,6 +769,10 @@ class PhilipsTV(object):
             return r
 
     async def getChannelId(self):
+        if not self.channels:
+            self.channel = None
+            return None
+
         if self.api_version >= 5:
             r = cast(Optional[ActivitesTVType], await self.getReq("activities/tv"))
         else:
@@ -910,6 +978,24 @@ class PhilipsTV(object):
                 return True
         return False
 
+    async def getHueLampPower(self):
+        if self.json_feature_supported("ambilight", "Hue"):
+            r = await self.getReq("HueLamp/power")
+            if r and "power" in r:
+                self.huelamp_power = cast(str, r["power"])
+            else:
+                self.huelamp_power = None
+            return r
+        return None
+
+    async def setHueLampPower(self, state):
+        if self.json_feature_supported("ambilight", "Hue"):
+            data = {"power": state}
+            if await self.postReq("HueLamp/power", data) is not None:
+                self.huelamp_power = state
+                return True
+        return False
+
     async def setApplication(self, intent: ApplicationIntentType):
         if self.json_feature_supported("activities", "intent"):
             data = {"intent": intent}
@@ -1195,9 +1281,74 @@ class PhilipsTV(object):
             return {
                 translation["string_id"]: translation["string_translation"]
                 for translation in res["translations"]
+                if translation["string_translation"]
             }
         return None
 
+    async def getMenuItemsSettingsStructure(self, force=False) -> Optional[MenuItemsSettingsStructure]:
+        if self.json_feature_supported("menuitems", "Setup_Menu") or force:
+            r = cast(Optional[MenuItemsSettingsStructure],
+                    await self.getReq("menuitems/settings/structure")
+            )
+            self.settings = r
+            return r
+
+
+    async def getMenuItemsSettingsCurrent(self, node_ids: List[int], force=False) -> Optional[MenuItemsSettingsCurrent]:
+        if not node_ids:
+            return None
+
+        if self.json_feature_supported("menuitems", "Setup_Menu") or force:
+            post: MenuItemsSettingsCurrentPost = {
+                "nodes": [
+                    {
+                        "nodeid": node_id
+                    }
+                    for node_id in node_ids
+                ] 
+            }
+            r = cast(Optional[MenuItemsSettingsCurrent],
+                     await self.postReq("menuitems/settings/current", cast(dict, post))
+            )
+            return r
+
+    async def getMenuItemsSettingsCurrentValue(self, node_ids: List[int], force=False) -> Dict[int, Optional[MenuItemsSettingsCurrentValueValue]]:
+            data = await self.getMenuItemsSettingsCurrent(node_ids, force=force)
+            res: Dict[int, Optional[MenuItemsSettingsCurrentValueValue]] = {}
+            if data:
+                for value in data.get("values", {}):
+                    value_value = value["value"]
+
+                    # Sometimes the API places the data field outside of the value field.
+                    # This seem faulty, so let's move it in.
+                    if data := value.get("data"):
+                        value_value["data"] = data
+
+                    res[value["value"].get("Nodeid")] = value_value
+            for node_id in node_ids:
+                if node_id not in res:
+                    res[node_id] = None
+            return res
+
+    async def postMenuItemsSettingsUpdate(self, post: MenuItemsSettingsUpdate, force=False):
+        if self.json_feature_supported("menuitems", "Setup_Menu") or force:
+            return await self.postReq("menuitems/settings/update", cast(dict, post))
+
+    async def postMenuItemsSettingsUpdateData(self, value: Dict[int, MenuItemsSettingsValueData], force=False):
+        post = {
+            "values": [
+                {
+                    "value": {
+                        "Nodeid": node_id,
+                        "data": data
+                    },
+                }
+                for node_id, data in value.items()
+            ]
+        }
+        return await self.postMenuItemsSettingsUpdate(cast(MenuItemsSettingsUpdate, post), force=force)
+        
+
     async def notifyChange(self, timeout=TIMEOUT_NOTIFYREAD):
         """Start a http connection waiting for changes."""
         if not self.notify_change_supported:
@@ -1211,6 +1362,7 @@ class PhilipsTV(object):
                 "audio/volume": self.audio_volume,
                 "context": self.context,
                 "screenstate": {"screenstate": self.screenstate},
+                "menuitems/settings/version": {"version": self.settings_version}
             }
         }
 
@@ -1244,6 +1396,10 @@ class PhilipsTV(object):
 
             if "screenstate" in result:
                 self.screenstate = result["screenstate"]["screenstate"]
+
+            if "menuitems/settings/version" in result:
+                self.settings_version = result["menuitems/settings/version"]["version"]
+
             return True
 
         if result is None:
