@@ -2,6 +2,8 @@ import curses
 import platform
 import json
 import sys
+
+from setuptools import find_namespace_packages
 from . import PhilipsTV
 import asyncio
 from ast import literal_eval
@@ -18,10 +20,10 @@ async def monitor_run(stdscr, tv: PhilipsTV):
 
         if tv.applications:
             for app in tv.applications.values():
-                if app["intent"] == tv.application:
-                    return app["label"]
+                if app.get("intent") == tv.application:
+                    return app.get("label")
 
-        return f"{tv.application['component']['className']}"
+        return tv.application.get("component", {}).get("className")
 
     while True:
 
@@ -30,10 +32,15 @@ async def monitor_run(stdscr, tv: PhilipsTV):
         if tv.source_id:
             stdscr.addstr(1, 0, await tv.getSourceName(tv.source_id))
 
+        stdscr.addstr(3, 15, "Menu Version")
+        if tv.channel_id:
+            stdscr.addstr(4, 15, str(tv.settings_version))
+
         stdscr.addstr(0, 15, "Channel")
         if tv.channel_id:
             stdscr.addstr(1, 15, await tv.getChannelName(tv.channel_id))
 
+
         stdscr.addstr(0, 30, "Application")
         if tv.application:
             stdscr.addstr(1, 30, get_application_name())
@@ -43,7 +50,7 @@ async def monitor_run(stdscr, tv: PhilipsTV):
             stdscr.addstr(1, 45, tv.context.get("level1", ""))
             stdscr.addstr(2, 45, tv.context.get("level2", ""))
             stdscr.addstr(3, 45, tv.context.get("level3", ""))
-            stdscr.addstr(3, 45, tv.context.get("data", ""))
+            stdscr.addstr(4, 45, tv.context.get("data", ""))
 
         def print_pixels(side, offset_y, offset_x):
             stdscr.addstr(offset_y, offset_x, "{}".format(side))
@@ -67,7 +74,9 @@ async def monitor_run(stdscr, tv: PhilipsTV):
                 print_pixels("bottom", offset_y, 45)
 
         stdscr.refresh()
-        await tv.update()
+        if not await tv.notifyChange():
+            await tv.update()
+
         key = stdscr.getch()
 
         if key == ord("q"):
@@ -166,11 +175,24 @@ async def main():
     strings.add_argument("--country", help="Country code to request")
     strings.add_argument("--variant", help="Variant")
 
+    settings = subparsers.add_parser("settings", help="Get setting")
+
+    settings_commands = settings.add_subparsers(help="settings commands", dest="settings_command")
+    settings_get = settings_commands.add_parser("get", help="Get current setting")
+    settings_get.add_argument("node_id", help="node id to request", type=int)
+
+    settings_structure = settings_commands.add_parser("structure", help="Get settings structure")
+
+    settings_set = settings_commands.add_parser("set", help="Post a settings value")
+    settings_set.add_argument("node_id", help="node id to request", type=int)
+    settings_set.add_argument("data", help="Json data to post", type=json.loads)
 
     markdown = subparsers.add_parser("markdown", help="Print markdown for commandline")
+    notify = subparsers.add_parser("notify", help="Look at notify changes, use with debug")
+
 
     args = parser.parse_args()
-    logging.basicConfig(level=args.debug and logging.DEBUG or logging.INFO)
+    logging.basicConfig(level=logging.DEBUG if args.debug else logging.INFO)
     haphilipsjs.TIMEOUT = 60.0
     tv = haphilipsjs.PhilipsTV(
         args.host,
@@ -179,7 +201,13 @@ async def main():
         password=args.password,
         secured_transport=args.secured_transport,
     )
+    try:
+        await run(args, parser, tv)
+    finally:
+        await tv.aclose()
 
+
+async def run(args, parser, tv: PhilipsTV):
     if args.command == "status":
         await tv.update()
 
@@ -222,7 +250,7 @@ async def main():
                 "Applications: {}".format(
                     ", ".join(
                         [
-                            application["label"] or "None"
+                            application.get("label") or "None"
                             for application in tv.applications.values()
                         ]
                     )
@@ -243,6 +271,7 @@ async def main():
                 tv.ambilight_current_configuration
             )
         )
+        print("Ambilight+Hue State: {}".format(tv.huelamp_power))
 
     elif args.command == "ambilight":
         if args.ambilight_mode:
@@ -272,23 +301,37 @@ async def main():
 
     elif args.command == "get":
         res = await tv.getReq(args.path)
-        json.dump(res, sys.stdout, indent=2)
+        json.dump(res, sys.stdout, indent=2, ensure_ascii=False)
 
     elif args.command == "post":
         res = await tv.postReq(args.path, literal_eval(args.data))
-        json.dump(res, sys.stdout, indent=2)
+        json.dump(res, sys.stdout, indent=2, ensure_ascii=False)
 
     elif args.command == "strings":
         res = await tv.getStrings(args.string_ids, args.language, args.country, args.variant)
-        json.dump(res, sys.stdout, indent=2)
+        json.dump(res, sys.stdout, indent=2, ensure_ascii=False)
 
+    elif args.command == "settings":
+        if args.settings_command == "structure":
+            res = await tv.getMenuItemsSettingsStructure(force=True)
+            json.dump(res, sys.stdout, indent=2, ensure_ascii=False)
+        elif args.settings_command == "get":
+            res = await tv.getMenuItemsSettingsCurrentValue([args.node_id], force=True)
+            json.dump(res[args.node_id], sys.stdout, indent=2, ensure_ascii=False)
+            print()
+        elif args.settings_command == "set":
+            res = await tv.postMenuItemsSettingsUpdateData({args.node_id: args.data}, force=True)
+            json.dump(res, sys.stdout, indent=2, ensure_ascii=False)
+            print()
     elif args.command == "markdown":
         import argmark
 
         argmark.md_help(parser)
 
-    await tv.aclose()
-
+    elif args.command == "notify":
+        await tv.update()
+        while await tv.notifyChange() is not None:
+            pass
 
 if __name__ == "__main__":
     asyncio.run(main())
